<html>
<head>
    <title>{{journal_title}}</title>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1, user-scalable=no" />
    <link rel="stylesheet" href="css/journal.css" />
    <link rel="stylesheet" href="css/responsive.css" />
</head>
<body>

<div id="page-wrapper">

    <!-- Header -->
    <header id="header">
        <div id="header-wrapper">
            <h1 id="logo"></h1>
            <nav id="nav">
                <ul>
                    <li class="current"><a href="index.html">Index</a></li>
                </ul>
            </nav>
        </div>
    </header>

    <!-- Main -->
    <article id="main">

        <header id="index_header" class="special container">
        </header>

        <section class="wrapper style4 container">
            <!-- Content -->
            <div class="content">
                <section>
                    <div id="index_page" class="index-page">

                        <!-- cover picture -->

                        <!-- Table of contents -->
                        <div id="toc_container" class="toc_container">

                        </div>

                    </div>

                </section>
            </div>

        </section>
    </article>

    <!-- Footer -->
    <footer id="footer">
        <ul class="copyright">
            <li>Exported using the
                <strong><a href="https://github.com/JohnHenrySplitMyHeart/bikesanity">BikeSanity</a></strong> cycle journal exporter. All content copyright the author
            </li>
        </ul>
    </footer>

</div>
</body>
</html>
