<html>
<head>
    <title>{{page_title}}</title>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1, user-scalable=no" />
    <link rel="stylesheet" href="css/journal.css" />
    <link rel="stylesheet" href="css/responsive.css" />
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.3.1/leaflet.css" />
</head>
<body>
<script src="https://cdnjs.cloudflare.com/ajax/libs/leaflet/1.3.1/leaflet.js"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/leaflet-gpx/1.4.0/gpx.min.js"></script>
<script src="js/maps.js"></script>

<div id="page-wrapper">

    <!-- Header -->
    <header id="header">
        <div id="header-wrapper">
            <h1 id="logo"></h1>
            <nav id="nav">
                <ul>
                    <li class="current"><a href="index.html">Index</a></li>
                    <!-- Navigation links -->
                </ul>
            </nav>
        </div>
    </header>

    <!-- Main -->
    <article id="main">

        <header id="page_header" class="special container">
            <!-- Page title -->
        </header>

        <section class="wrapper style4 container">
            <!-- Content -->
            <div class="content">
                <section>
                    <div id="journal_page" class="journal-page">

                        <!-- Page content -->

                    </div>
                </section>
            </div>

        </section>
    </article>

    <!-- Footer -->
    <footer id="footer">
        <ul class="copyright">
            <li>Exported using the
                <strong><a href="https://github.com/JohnHenrySplitMyHeart/bikesanity">BikeSanity</a></strong> cycle journal exporter. All content copyright the author
            </li>
        </ul>
    </footer>

</div>
</body>
</html>
