/*
	Adapted from Twenty by HTML5 UP
	html5up.net | @ajlkn
	Free for personal and commercial use under the CCA 3.0 license (html5up.net/license)
*/


/* Normal */
@media screen and (max-width: 1280px) {

  /* Basic */

  body, input, select, textarea {
    font-size: 13pt;
    letter-spacing: 0.025em;
    line-height: 1.65em;
  }

  h1, h2, h3, h4, h5, h6 {
    line-height: 1.5em;
  }

  /* Section/Article */

  header.major {
    padding-bottom: 1.5em;
  }

  footer.major {
    padding-top: 2em;
  }

  /* Wrapper */

  .wrapper {
    margin-bottom: 4em;
    padding: 4em 3em;
  }

  .wrapper.style4 {
    padding: 3em;
  }

  /* Header */

  #header nav ul li {
    margin-left: 1em;
  }

  /* Banner */

  #banner {
    background-attachment: scroll;
  }

  /* CTA */

  #cta {
    padding: 4em;
    background-attachment: scroll;
  }

  /* Footer */

  #footer {
    padding: 1em;
  }

}

/* Narrow */

@media screen and (max-width: 980px) {

  /* Basic */

  body, input, select, textarea {
    font-size: 13pt;
    letter-spacing: 0.025em;
    line-height: 1.5em;
  }

  /* Section/Article */

  header br {
    display: none;
  }

  header.major {
    padding-bottom: 1em;
  }

  header.special {
    padding-left: 2.5em;
    padding-right: 2.5em;
  }

  footer.major {
    padding-top: 1.5em;
  }

  /* Wrapper */

  .wrapper {
    margin-bottom: 3em;
    padding: 3em 2.5em;
  }

  .wrapper.special br {
    display: none;
  }

  .wrapper.style1 {
    padding: 0 2.5em;
  }

  .wrapper.style2 {
    background-size: 15em;
  }

  .wrapper.style4 {
    padding: 2.5em;
  }

  span.collapses {
    display: none !important;
  }

  /* Banner */

  #banner {
    background-image: url("images/light-bl.svg"), url("images/light-br.svg"), url("images/overlay.png"), url("../img/banner.jpeg");
    background-size: 15em, 15em, auto, cover;
  }

  /* Main */

  #main {
    background-size: 15em;
  }

  /* CTA */

  #cta {
    background-size: 15em, 15em, auto, cover;
    padding: 3em;
  }

}

/* Narrower */

#navPanel, #navButton {
  display: none;
}

@media screen and (max-width: 840px) {

  /* Basic */

  html, body {
    overflow-x: hidden;
  }

  header.major {
    padding-bottom: 0.25em;
  }

  header.special {
    margin-bottom: 4em;
    padding-top: 3em;
  }

  header.special:before, header.special:after {
    width: 40%;
  }

  header.special h2 + p {
    padding-top: 1.25em;
  }

  /* Section/Article */

  section {
    margin: 1em 0 1em 0;
  }

  section:first-child {
    margin-top: 0;
  }

  /* Button */

  input[type="button"].small,
  input[type="submit"].small,
  input[type="reset"].small,
  button.small,
  .button.small {
    font-size: 0.8em;
    min-width: 18em;
    padding: 0.75em 0;
  }

  /* Featured Icons */

  ul.featured-icons {
    margin: 0;
  }

  ul.featured-icons li {
    display: inline-block;
    float: none;
    width: auto;
  }

  ul.featured-icons li .icon {
    font-size: 4em;
    width: 1.25em;
  }

  /* Buttons */

  ul.buttons li {
    display: block;
    padding: 1em 0 0 0;
  }

  /* Header */

  #logo {
    display: none;
  }

  .logo {
    display: none;
  }

  .collapses {
    display: none !important;
  }

  li .collapses  {
    display: none !important;
  }

  /* Banner */

  #banner {
    background-image: url("images/light-bl.svg"), url("images/light-br.svg"), url("images/overlay.png"), url("../img/banner.jpeg");
    margin: 0;
  }

  /* Wrapper */

  .wrapper.special-alt {
    text-align: center;
  }

  .wrapper.style4 {
    padding-bottom: 3em;
  }

  /* Main */

  #main {
    padding: 5em 0;
  }

  #main .sidebar {
    border-top: solid 1px rgba(124, 128, 129, 0.1);
    padding-top: 3em;
  }

  #main .sidebar section {
    border-top: 0;
    padding-top: 0;
  }

  body.index #main {
    padding-top: 4.5em;
  }

  /* CTA */

  #cta {
    margin: 0;
  }

  /* Footer */

  #footer {
    padding: 1em 1.5em;
  }

  /* Nav */

  #page-wrapper {
    -moz-backface-visibility: hidden;
    -webkit-backface-visibility: hidden;
    -ms-backface-visibility: hidden;
    backface-visibility: hidden;
    -moz-transition: -moz-transform 0.5s ease;
    -webkit-transition: -webkit-transform 0.5s ease;
    -ms-transition: -ms-transform 0.5s ease;
    transition: transform 0.5s ease;
    padding-bottom: 1px;
  }

  #navButton {
    -moz-backface-visibility: hidden;
    -webkit-backface-visibility: hidden;
    -ms-backface-visibility: hidden;
    backface-visibility: hidden;
    -moz-transition: -moz-transform 0.5s ease;
    -webkit-transition: -webkit-transform 0.5s ease;
    -ms-transition: -ms-transform 0.5s ease;
    transition: transform 0.5s ease;
    display: block;
    height: 60px;
    left: 0;
    position: fixed;
    top: 0;
    width: 100%;
    z-index: 10001;
  }

  #navButton .toggle {
    text-decoration: none;
    height: 60px;
    left: 0;
    position: absolute;
    text-align: center;
    top: 0;
    width: 100%;
    border: 0;
    outline: 0;
  }

  #navButton .toggle:before {
    -moz-osx-font-smoothing: grayscale;
    -webkit-font-smoothing: antialiased;
    display: inline-block;
    font-style: normal;
    font-variant: normal;
    text-rendering: auto;
    line-height: 1;
    text-transform: none !important;
    font-family: 'Font Awesome 5 Free';
    font-weight: 900;
  }

  #navButton .toggle:before {
    color: #fff;
    content: '\f0c9';
    font-size: 1em;
    height: 40px;
    left: 10px;
    line-height: 40px;
    opacity: 0.5;
    position: absolute;
    top: 11px;
    width: 60px;
    z-index: 1;
  }

  #navButton .toggle:after {
    background: rgba(163, 169, 170, 0.75);
    border-radius: 2px;
    content: '';
    height: 40px;
    left: 10px;
    position: absolute;
    top: 10px;
    width: 60px;
  }

  #navPanel {
    -moz-backface-visibility: hidden;
    -webkit-backface-visibility: hidden;
    -ms-backface-visibility: hidden;
    backface-visibility: hidden;
    -moz-transform: translateX(-275px);
    -webkit-transform: translateX(-275px);
    -ms-transform: translateX(-275px);
    transform: translateX(-275px);
    -moz-transition: -moz-transform 0.5s ease;
    -webkit-transition: -webkit-transform 0.5s ease;
    -ms-transition: -ms-transform 0.5s ease;
    transition: transform 0.5s ease;
    display: block;
    height: 100%;
    left: 0;
    overflow-y: auto;
    position: fixed;
    top: 0;
    width: 275px;
    z-index: 10002;
    background: #1c2021;
    color: #fff;
    font-size: 0.8em;
    letter-spacing: 0.075em;
    text-transform: uppercase;
    padding: 0.25em 0.75em 1em 0.75em;
  }

  #navPanel .link {
    border: 0;
    border-top: solid 1px rgba(255, 255, 255, 0.05);
    color: inherit;
    display: block;
    height: 3em;
    line-height: 3em;
    opacity: 0.75;
    text-decoration: none;
    padding-left: 1em;
  }

  #navPanel .link.depth-0 {
    font-weight: 900;
  }

  #navPanel .link:first-child {
    border-top: 0;
  }

  #navPanel .indent-1 {
    display: inline-block;
    width: 1em;
  }

  #navPanel .indent-2 {
    display: inline-block;
    width: 2em;
  }

  #navPanel .indent-3 {
    display: inline-block;
    width: 3em;
  }

  #navPanel .indent-4 {
    display: inline-block;
    width: 4em;
  }

  #navPanel .indent-5 {
    display: inline-block;
    width: 5em;
  }

  body.navPanel-visible #page-wrapper {
    -moz-transform: translateX(275px);
    -webkit-transform: translateX(275px);
    -ms-transform: translateX(275px);
    transform: translateX(275px);
  }

  body.navPanel-visible #navButton {
    -moz-transform: translateX(275px);
    -webkit-transform: translateX(275px);
    -ms-transform: translateX(275px);
    transform: translateX(275px);
  }

  body.navPanel-visible #navPanel {
    -moz-transform: translateX(0);
    -webkit-transform: translateX(0);
    -ms-transform: translateX(0);
    transform: translateX(0);
  }

}

/* Mobile */

@media screen and (max-width: 736px) {

  /* Basic */

  body {
    min-width: 320px;
  }

  h2 {
    font-size: 1.25em;
    letter-spacing: 0.1em;
  }

  h3 {
    font-size: 1em;
    letter-spacing: 0.025em;
  }

  p {
    text-align: justify;
  }

  /* Section/Article */

  header {
    text-align: center;
  }

  header.major {
    padding-bottom: 0;
  }

  header.special {
    margin-bottom: 3em;
    padding-left: 1.5em;
    padding-right: 1.5em;
  }

  header.special:before, header.special:after {
    width: 38%;
  }

  header.special .icon {
    font-size: 0.75em;
    top: 1.5em;
  }

  header p {
    text-align: center;
  }

  footer.major {
    padding-top: 0;
  }

  /* Icon */

  .icon.circle {
    font-size: 1em;
  }

  /* Icons */

  ul.icons li {
    padding-left: 0.25em;
  }

  /* Featured Icons */

  ul.featured-icons li .icon {
    width: 1.1em;
  }

  /* Buttons */

  ul.buttons {
    text-align: center;
  }

  /* Wrapper */

  .wrapper {
    margin-bottom: 2.5em;
    padding: 2.25em 1.5em;
  }

  .wrapper.special br {
    display: none;
  }

  .wrapper.style1 {
    padding: 0 1.5em;
  }

  .wrapper.style2 {
    background-size: 10em;
    padding: 2.25em 1.5em;
  }

  .wrapper.style4 {
    background-size: 10em;
    padding: 1.5em 1.5em 3em 1.5em;
  }

  /* Banner */

  #banner {
    background-image: url("images/light-bl.svg"), url("images/light-br.svg"), url("images/overlay.png"), url("../img/banner.jpeg");
    background-size: 10em, 10em, auto, cover;
    padding: 6em 0;
  }

  #banner .inner {
    background: none;
    display: block;
    padding: 0 1.5em;
  }

  #banner .inner header h2 {
    font-size: 1.5em;
  }

  #banner .inner p {
    text-align: center;
  }

  #banner .inner br {
    display: none;
  }

  /* Main */

  #main {
    background-size: 10em;
    padding: 3.5em 0 2.5em 0;
  }

  body.index #main {
    padding: 2.5em 0 0 0;
  }

  body.contact #main {
    padding-bottom: 0;
  }

  /* CTA */

  #cta {
    background-size: 10em, 10em, auto, cover;
    padding: 3em 1.5em;
  }

  /* Footer */

  #footer {
    padding: 1em 1.5em;
  }

  #footer .copyright li {
    display: block;
    margin: 1em 0 0 0;
    padding: 0;
    border: 0;
  }

  /* Nav */

  #navButton .toggle:before {
    top: 8px;
    left: 8px;
    width: 50px;
    height: 34px;
    line-height: 34px;
  }

  #navButton .toggle:after {
    top: 8px;
    left: 8px;
    width: 50px;
    height: 34px;
  }
}
